/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:convert';

import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:welltested/src/extensions/annotation_ext.dart';
import 'package:welltested/src/models/variable.dart';

abstract class UnitTestRequest {
  final CompilationUnit compilationUnit;
  final Declaration currentMethod;
  final String functionName;
  final String fileImportPath;

  final List<String> simpleModels;
  final List<ClassElement> complexModels;
  final List<String> staticMethods;
  final List<String> nestedMethods;
  final List<List<SimpleIdentifier>> callFlows;
  final List<Variable> variableDeclarations;
  bool forceGenerate;

  UnitTestRequest({
    required this.compilationUnit,
    required this.currentMethod,
    required this.functionName,
    required this.fileImportPath,
    required this.simpleModels,
    required this.complexModels,
    required this.staticMethods,
    required this.nestedMethods,
    required this.callFlows,
    required this.variableDeclarations,
    this.forceGenerate = false,
  });

  String get fileCode => compilationUnit.toSource();

  List<String> get testcases {
    List<String> casesFromAnnotation = [];

    currentMethod.metadata.testcasesAnnotations.forEach((e) {
      final testcasesArgument = e.arguments?.arguments.first as ListLiteral;
      if (testcasesArgument.elements.isNotEmpty) {
        casesFromAnnotation = testcasesArgument.elements
            .map((element) => (element as StringLiteral).stringValue ?? "")
            .toList();
      }
    });

    return casesFromAnnotation;
  }

  List<String> get flowPointers {
    List<String> pointers = [];
    callFlows.forEach((flow) {
      if (flow.length > 1) {
        for (int i = 1; i < flow.length; i++) {
          final parameterField = flow[i].staticElement?.name;
          final parameterType =
              flow[i].staticType?.getDisplayString(withNullability: true);
          final parameterParent =
              flow[i - 1].staticType?.getDisplayString(withNullability: false);
          pointers
              .add("$parameterParent().$parameterField return $parameterType");
        }
      }
    });
    return pointers;
  }

  Map<String, dynamic> get data {
    final data = {
      "function_name": functionName,
      "file_import_path": fileImportPath,
      "models": jsonEncode(simpleModels + staticMethods),
      "complex_models": jsonEncode(complexModels.map((e) => e.name).toList()),
      "flow_pointers": jsonEncode(flowPointers),
      "nested_methods": jsonEncode(nestedMethods),
      "testcases": jsonEncode(testcases),
      "force_generate": jsonEncode(forceGenerate),
    };
    return data;
  }

  String codeInScrutity({bool excludeAnnotations = true}) {
    throw Exception('Override for this method is missing');
  }

  Map<String, dynamic> toJson() {
    throw Exception('Override for this method is missing');
  }
}
