/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:welltested/src/extensions/annotation_ext.dart';
import 'package:welltested/src/models/test_request/base_test_request.dart';
import 'package:welltested/src/models/variable.dart';

class ExtensionTestRequest extends UnitTestRequest {
  final String extensionName;
  final ExtensionDeclaration extensionNode;
  final Map<String, MethodDeclaration> methodAstNodes;
  ExtensionTestRequest(
      {required CompilationUnit compilationUnit,
      required Declaration currentMethod,
      required String functionName,
      required String fileImportPath,
      required this.extensionName,
      required this.extensionNode,
      required this.methodAstNodes,
      required List<String> simpleModels,
      required List<ClassElement> complexModels,
      required List<String> staticMethods,
      required List<String> nestedMethods,
      required List<List<SimpleIdentifier>> callFlows,
      required List<Variable> variableDeclarations})
      : super(
            compilationUnit: compilationUnit,
            currentMethod: currentMethod,
            functionName: functionName,
            fileImportPath: fileImportPath,
            simpleModels: simpleModels,
            complexModels: complexModels,
            staticMethods: staticMethods,
            nestedMethods: nestedMethods,
            callFlows: callFlows,
            variableDeclarations: variableDeclarations);
  @override
  String codeInScrutity({bool excludeAnnotations = true}) {
    String code = fileCode;
    //remove all other functions in the class except the ones nested in this method.
    methodAstNodes.entries.forEach((method) {
      if (method.key != functionName && !nestedMethods.contains(method.key)) {
        code =
            code.replaceAll(methodAstNodes[method.key]?.toSource() ?? '', '');
      }
    });

    String methodCode = currentMethod.toSource();
    if (excludeAnnotations) {
      // remove all annotations from class
      extensionNode.metadata.packageAnnotations.forEach((element) {
        final annotationCode = element.toSource();
        code = code.replaceAll(annotationCode, '');
      });

      /// remove all annotations from current method.
      currentMethod.metadata.packageAnnotations.forEach((element) {
        final annotationCode = element.toSource();
        code = code.replaceAll(annotationCode, '');
      });
    }
    for (var i = 0; i < variableDeclarations.length; i++) {
      // before: final value = getValue()
      // after: final String value = getValue()
      final Variable variable = variableDeclarations[i];
      if (!methodCode.contains(variable.variableWithType)) {
        methodCode =
            methodCode.replaceFirst(variable.name, variable.variableWithType);
      }
    }
    code = code.replaceAll(currentMethod.toSource(), methodCode);
    return code;
  }

  @override
  Map<String, dynamic> toJson() {
    final jsonRequest = data;
    jsonRequest['code'] = codeInScrutity();
    //TODO: Send extension name instead
    jsonRequest['class_name'] = extensionName;
    return jsonRequest;
  }
}
