/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:welltested/src/extensions/annotation_ext.dart';
import 'package:welltested/src/models/test_request/base_test_request.dart';
import 'package:welltested/src/models/variable.dart';

class GlobalTestRequest extends UnitTestRequest {
  GlobalTestRequest(
      {required CompilationUnit compilationUnit,
      required String functionName,
      required Declaration currentMethod,
      required String fileImportPath,
      required List<String> simpleModels,
      required List<ClassElement> complexModels,
      required List<String> staticMethods,
      required List<String> nestedMethods,
      required List<List<SimpleIdentifier>> callFlows,
      required List<Variable> variableDeclarations})
      : super(
            compilationUnit: compilationUnit,
            functionName: functionName,
            fileImportPath: fileImportPath,
            currentMethod: currentMethod,
            simpleModels: simpleModels,
            complexModels: complexModels,
            staticMethods: staticMethods,
            nestedMethods: nestedMethods,
            callFlows: callFlows,
            variableDeclarations: variableDeclarations);
  @override
  String codeInScrutity({bool excludeAnnotations = true}) {
    String code = '';
    compilationUnit.directives.forEach((directive) => directive
            is ImportDirective
        ? code = code + directive.toSource() + '\n' //add all import directives
        : null);
    code = code + currentMethod.toSource();
    if (excludeAnnotations) {
      /// remove all welltested package related annotation
      currentMethod.metadata.packageAnnotations.forEach((element) {
        final annotationCode = element.toSource();
        code = code.replaceAll(annotationCode, '');
      });
    }

    return code;
  }

  @override
  Map<String, dynamic> toJson() {
    final jsonRequest = data;
    jsonRequest['code'] = codeInScrutity();
    return jsonRequest;
  }
}
