
import 'dart:convert';
import 'dart:io';

import 'package:path/path.dart' as path;

class SharedCache {
  static const _cacheDirectoryName = 'welltested';
  static const _cacheFileName = 'welltested_cache.json';

  static String _getCacheFolderPath() {
    final pubCachePath = path.join(Platform.environment['PUB_CACHE'] ?? path.join(Platform.environment['HOME']!, '.pub-cache'));
    return path.join(pubCachePath,'global_packages', _cacheDirectoryName);
  }

  static String _getCacheFilePath() => path.join(_getCacheFolderPath(),_cacheFileName);

  static Future<Null> init() async {
    final cacheFolderPath = _getCacheFolderPath();
    final cacheFilePath = _getCacheFilePath();
    if (!await Directory(cacheFolderPath).exists()) {
      await Directory(cacheFolderPath).create(recursive:true);
    }

    if (!await File(cacheFilePath).exists()) {
      await File(cacheFilePath).create(recursive: true);
      await File(cacheFilePath).writeAsString(jsonEncode({}));
    }
  }

  static Future<Map<String, dynamic>> readCache() async {
    final cacheFile = await File(_getCacheFilePath()).readAsString();
    return jsonDecode(cacheFile);
  }

  static Future<Null> updateCache(Map<String, dynamic> newData) async {
    final cacheData = await readCache();
    for (var key in newData.keys) {
      cacheData[key] = newData[key];
    }
    await File(_getCacheFilePath()).writeAsString(jsonEncode(cacheData));
  }
  
  static Future<String?> getCachedKey()async{
    final cacheData = await readCache();
    return cacheData['WELLTESTED_API'];
  }

  static Future<void> updateKey(String key) async {
    await updateCache({'WELLTESTED_API': key});
  }
}