/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:io';

import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:path/path.dart' as p;
import 'package:welltested/src/utilities/logging.dart';
import 'package:welltested_annotation/welltested_annotation.dart';

Future<AstNode> getResolvedAstNodeForElement(
  Element element,
) async {
  final resolvedLib =
      await element.session!.getResolvedLibraryByElement(element.library!);
  final astNode = (resolvedLib as ResolvedLibraryResult)
      .getElementDeclaration(element)!
      .node;
  return astNode;
}

bool isPrivate(String name) {
  return name.startsWith("_");
}

bool isAnnotationPresent(Element element) {
  String annotationType = Welltested().runtimeType.toString();
  for (final annotation in element.metadata) {
    if (annotation.element!.displayName == annotationType) {
      return true;
    }
  }
  return false;
}

bool isDartProject(String path) {
  final file = File(p.join(path, "pubspec.yaml"));
  return file.existsSync();
}

bool verify_two_true(List<bool> vars) {
  int trueCount = 0;

  for (bool variable in vars) {
    if (variable) trueCount++;
    if (trueCount > 1) return true;
  }

  return false;
}

runCommand(String command, List<String> arguments) async {
  try {
    final result = await Process.run(command, arguments);
  if (result.exitCode != 0) {
    throw Exception(result.stderr);
  }
  } catch (e) {
    wtLog.error("Error running command: $command $arguments");
    rethrow;
  }
}