/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:welltested/src/models/variable.dart';
import 'package:welltested_annotation/welltested_annotation.dart';

class FunctionBodyVisitor extends RecursiveAstVisitor {
  final List<ClassElement> classElements = [];
  final List<List<SimpleIdentifier>> callFlows = [];
  final List<Variable> updateVariableNameNType = [];
  final List<String> nestedMethods = [];
  final List<MethodElement> staticMethods = [];

  void addToClassElements(ClassElement model) {
    if (!classElements.map((e) => e.name).contains(model.name)) {
      classElements.add(model);
    }
  }

  List<SimpleIdentifier> getListOfPropAccess(PropertyAccess node) {
    final List<SimpleIdentifier> flows = [];
    for (final entity in node.childEntities) {
      if (entity is PrefixedIdentifier) {
        flows.addAll(extractPrefixedIdentifier(entity));
      }
      if (entity is SimpleIdentifier) {
        flows.add(entity);
      }
      if (entity is PropertyAccess) {
        flows.addAll(getListOfPropAccess(entity));
      }
    }

    return flows;
  }

  List<SimpleIdentifier> extractPrefixedIdentifier(
      PrefixedIdentifier prefixedIdentifier) {
    final List<SimpleIdentifier> flows = [];
    for (final entity in prefixedIdentifier.childEntities) {
      if (entity is PrefixedIdentifier) {
        flows.addAll(extractPrefixedIdentifier(prefixedIdentifier));
      }
      if (entity is SimpleIdentifier) {
        flows.add(entity);
      }
      if (entity is PropertyAccess) {
        flows.addAll(getListOfPropAccess(entity));
      }
    }
    return flows;
  }

  @override
  visitPropertyAccess(PropertyAccess node) {
    final flow = getListOfPropAccess(node);
    callFlows.add(flow);
  }

  @override
  visitVariableDeclarationList(VariableDeclarationList node) {
    for (final entity in node.childEntities) {
      if (entity is VariableDeclaration) {
        updateVariableNameNType.add(Variable(
            name: entity.declaredElement!.name,
            type: '${entity.declaredElement!.type}'));
      }
    }

    return super.visitVariableDeclarationList(node);
  }

  @override
  visitMethodInvocation(MethodInvocation node) {
    if (node.methodName.staticElement == null) return;
    if ((node.methodName.staticElement is MethodElement) &&
        (node.methodName.staticElement as MethodElement).isStatic) {
      //includes static and global methods.
      staticMethods.add(node.methodName.staticElement as MethodElement);
    }
    if (!(node.methodName.staticElement?.library?.isInSdk ?? false)) {
      nestedMethods.add(node.methodName.name);
    }

    // if(node.methodName.staticElement is MethodElement && node.methodName.staticElement.isstat)
    return super.visitMethodInvocation(node);
  }

  @override
  visitSimpleIdentifier(SimpleIdentifier node) {
    final staticElement = node.staticElement;

    if (staticElement is ClassElement &&
            !staticElement.library.isInSdk &&
            !staticElement.isPrivate
        //TODO: Add back again after upgrading to ^5.7.1
        // && !staticElement.isSealed
        ) {
      // filter out welltested and testcases models.
      if (staticElement.name == Welltested().runtimeType.toString() ||
          staticElement.name == Testcases([]).runtimeType.toString()) return;
      addToClassElements(staticElement);
    }
    return super.visitSimpleIdentifier(node);
  }
}
